#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const colors_1 = __importDefault(require("colors"));
const inquirer_1 = __importDefault(require("inquirer"));
const commander_1 = require("commander");
const child_process_1 = require("child_process");
const replace_in_file_1 = require("replace-in-file");
const dirExists = (projectName) => {
    if (fs_1.default.existsSync(path_1.default.join(process.cwd(), projectName))) {
        console.log(colors_1.default.yellow('\nDirectory already exists.'));
        process.exit(0);
    }
};
const templates = [
    'vanilla',
    'vanilla-ts',
    'react',
    'react-ts',
    'preact',
    'preact-ts',
    'next',
    'next-ts',
];
commander_1.program
    .option('-t, --template <template>', 'choose a template')
    .option('-y, --yarn', 'use yarn')
    .parse();
(async () => {
    let projectName;
    let template;
    let useNpm = true;
    if (!process.argv[2]) {
        const answers = await inquirer_1.default.prompt([
            {
                message: 'Name of your project:',
                name: 'projectName',
                default: 'my-twpx-app',
            },
            {
                message: 'Choose a template:',
                name: 'template',
                type: 'list',
                choices: templates.filter((t) => !t.includes('ts')),
                default: 'vanilla',
            },
        ]);
        const _template = answers['template'];
        const _answers = await inquirer_1.default.prompt([
            {
                message: 'Choose a variation:',
                name: 'template',
                type: 'list',
                choices: [_template, _template + '-ts'],
                default: _template,
            },
            {
                message: 'Install with:',
                name: 'pkgManager',
                type: 'list',
                choices: ['npm', 'yarn'],
                default: 'npm',
            },
        ]);
        projectName = answers['projectName'];
        template = _answers['template'];
        useNpm = _answers['pkgManager'] === 'npm';
        dirExists(projectName);
    }
    else {
        const options = commander_1.program.opts();
        projectName = process.argv[2];
        dirExists(projectName);
        if (!options.template) {
            console.log('\nProvide a template by using -t <template> or --template <template>\nAvailable templates:\n', `\n${templates.join('\n')}`);
            process.exit(1);
        }
        else if (!templates.includes(options.template)) {
            console.log('\nTemplate does not exist. Available templates:\n', `\n${templates.join('\n')}`);
            process.exit(1);
        }
        template = options.template;
        if (options.yarn)
            useNpm = false;
    }
    const npmTemplate = `npm init vite@latest ${projectName} -- --template ${template}`;
    const yarnTemplate = `yarn create vite ${projectName} --template ${template}`;
    const nextTsFlag = template === 'next-ts' ? '--ts' : '';
    const nextNpmTemplate = `npx create-next-app@latest ${projectName} ${nextTsFlag}`;
    const nextYarnTemplate = `yarn create next-app ${projectName} ${nextTsFlag}`;
    const usePkgManagerMsg = (pkg) => {
        return colors_1.default.gray(`\n→ Using ${pkg} to install packages...`);
    };
    const useNextTemplate = template.includes('next');
    if (!useNpm) {
        if (useNextTemplate) {
            (0, child_process_1.execSync)(nextYarnTemplate, { stdio: 'inherit' });
        }
        else {
            console.log(usePkgManagerMsg('yarn'));
            (0, child_process_1.execSync)(yarnTemplate, { stdio: 'inherit' });
        }
    }
    else {
        console.log(usePkgManagerMsg('npm'));
        if (useNextTemplate) {
            (0, child_process_1.execSync)(nextNpmTemplate, { stdio: 'inherit' });
        }
        else
            (0, child_process_1.execSync)(npmTemplate, { stdio: 'inherit' });
    }
    process.chdir(path_1.default.join(process.cwd(), projectName));
    try {
        console.log(colors_1.default.gray('\n→ Installing dependencies & Setting-up Tailwind CSS...'));
        let prefix = 'npm i';
        if (!useNpm)
            prefix = 'yarn add';
        (0, child_process_1.execSync)(`${prefix} -D tailwindcss@latest postcss@latest autoprefixer@latest`, { stdio: 'inherit' });
        (0, child_process_1.execSync)(`npx tailwindcss init -p`, { stdio: 'inherit' });
        console.log(colors_1.default.green.bold('\n[ Installed Successfully! ]'));
        console.log('\nGet Started, Run:'.gray);
        console.log(colors_1.default.yellow(`\ncd ${projectName}`));
        if (useNpm)
            console.log(colors_1.default.yellow('npm run dev'));
        else
            console.log(colors_1.default.yellow('yarn dev'));
    }
    catch (err) {
        console.error('An error occurred', err);
        process.exit(1);
    }
    const projectDir = process.cwd();
    const useTwConfig = () => {
        let content;
        if (useNextTemplate)
            content =
                "'./pages/**/*.{js,ts,jsx,tsx}', './components/**/*.{js,ts,jsx,tsx}', './src/**/*.{js,ts,jsx,tsx}'";
        else if (template.includes('vanilla'))
            content = "'./index.html'";
        else
            content = "'./index.html', './src/**/*.{js,ts,jsx,tsx}'";
        return `content: [${content}]`;
    };
    const useTwPath = () => {
        switch (template) {
            case 'next':
            case 'next-ts':
                return 'styles/globals.css';
            case 'vanilla':
                return 'style.css';
            case 'vanilla-ts':
                return 'src/style.css';
            default:
                return 'src/index.css';
        }
    };
    const tailwindConfig = {
        files: `${projectDir}/tailwind.config.${useNextTemplate ? 'js' : 'cjs'}`,
        from: /content: \[]/g,
        to: useTwConfig(),
    };
    try {
        await (0, replace_in_file_1.replaceInFile)(tailwindConfig);
        fs_1.default.writeFileSync(path_1.default.join(projectDir, useTwPath()), '@tailwind base;\n@tailwind components;\n@tailwind utilities;');
    }
    catch (err) {
        console.error('An error occurred:', err);
        process.exit(1);
    }
})();
